package it.unibas.esempihibernate.test.persistenza.es1mbidir;

import it.unibas.esempihibernate.es1mbidir.Automobile;
import it.unibas.esempihibernate.es1mbidir.Proprietario;
import it.unibas.esempihibernate.util.PersistenceManager;
import java.util.List;
import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;

public class Test1m extends TestCase {
 
    private static Log log = LogFactory.getLog(Test1m.class);

    private Automobile automobile;
    private Proprietario proprietario;
    
    protected void setUp() {
        proprietario = new Proprietario();
        proprietario.setCodiceFiscale("cftest1");
        proprietario.setNome("Proprietario di test");
        proprietario.setCittaDiResidenza("citta di test");
        proprietario.setAnnoPatente(2003);
        automobile = new Automobile();
        automobile.setTarga("test1");
        automobile.setModello("Modello di test");
        automobile.setCilindrata(2000);
    }
    
    public void testInserisciProprietario() {
        log.info("\n***************** Inizia test ************************");
        SessionFactory factory = PersistenceManager.getSessionFactory();
        Session session = null; 
        Transaction transaction = null;
        try {
            log.info("proprietario prima di save: " + proprietario);
            session = factory.openSession();
            transaction = session.beginTransaction();
            session.save(proprietario);
            transaction.commit();
            log.info("proprietario dopo save: " + proprietario);
            Query query = session.createQuery("select p from Proprietario p where codicefiscale = ?");
            query.setString(0, "cftest1");
            List proprietari = query.list();
            assertEquals(proprietari.size(), 1);
            Proprietario proprietario1 = (Proprietario)proprietari.get(0);
            log.info("proprietario1 dopo la query: " + proprietario1);
            assertEquals(proprietario1.getNome(), "Proprietario di test");
            assertEquals(proprietario1.getCittaDiResidenza(), "citta di test");
            assertEquals(proprietario1.getAnnoPatente(), 2003);
        } catch (HibernateException e) {
            log.error(e);
            if (transaction != null) {
                transaction.rollback();
            }
        } finally {
            elimina(proprietario, session);
            try {
                session.close();
            } catch (HibernateException e) {
                log.error(e);
            }
        }
    }
    
    public void testInserisciProprietarioErrore() {
        log.info("\n***************** Inizia test ************************");
        SessionFactory factory = PersistenceManager.getSessionFactory();
        Session session = null; 
        Transaction transaction = null;
        try {
            log.info("proprietario prima di save: " + proprietario);
            session = factory.openSession();
            transaction = session.beginTransaction();
            session.save(proprietario);
            transaction.commit();
            log.info("proprietario dopo save: " + proprietario);
            long id = proprietario.getId();
            Proprietario prop = new Proprietario();
            session.load(prop, new Long(id));
            fail();
        } catch (HibernateException e) {
            log.error(e);
            if (transaction != null) {
                transaction.rollback();
            }
        } finally {
            elimina(proprietario, session);
            try {
                session.close();
            } catch (HibernateException e) {
                log.error(e);
            }
        }
    }
    
    public void testInserisciAutomobile() {
        log.info("\n***************** Inizia test ************************");
        SessionFactory factory = PersistenceManager.getSessionFactory();
        Session session = null;
        Transaction transaction = null;
        try {
            session = factory.openSession();
            transaction = session.beginTransaction();
            automobile.setProprietario(proprietario);
            proprietario.addAutomobile(automobile);
            log.info("proprietario prima di save: " + proprietario);
            log.info("automobile prima di save: " + automobile);
            session.save(proprietario);
            session.save(automobile);
            transaction.commit();
            log.info("proprietario dopo save: " + proprietario);
            log.info("automobile dopo save: " + automobile);
            Query query = session.createQuery("select p.automobili from Proprietario p where codicefiscale = ?");
            query.setString(0, "cftest1");
            List automobili = query.list();
            assertEquals(automobili.size(), 1);
            Automobile automobile1 = (Automobile)automobili.get(0);
            log.info("automobile1 dopo la query: " + automobile1);
            assertEquals(automobile1.getTarga(), "test1");
            assertEquals(automobile1.getModello(), "Modello di test");
            assertEquals(automobile1.getCilindrata(), 2000);
        } catch (HibernateException e) {
            log.error(e);
            if (transaction != null) {
                transaction.rollback();
            }
        } finally {
            elimina(automobile, session);
            elimina(proprietario, session);
            try {
                session.close();
            } catch (HibernateException e) {
                log.error(e);
            }
        }
    }

    private void elimina(Proprietario proprietario, Session session) {
        Transaction transaction = null;
        try {
            transaction = session.beginTransaction();
            session.delete(proprietario);
            transaction.commit();
            log.info("delete: " + proprietario);
        } catch (HibernateException e) {
            log.error(e);
            if (transaction != null) {
                transaction.rollback();
            }
        } 
    }

    private void elimina(Automobile automobile, Session session) {
        Transaction transaction = null;
        try {
            transaction = session.beginTransaction();
            session.delete(automobile);
            transaction.commit();
            log.info("delete: " + automobile);
        } catch (HibernateException e) {
            log.error(e);
            if (transaction != null) {
                transaction.rollback();
            }
        } 
    }
}
